@@ -12,11 +12,22 @@ namespace Tomboy.Notebooks
 		static Gdk.Pixbuf notebookIcon;
 		static Gdk.Pixbuf newNotebookIcon;
 		
-		static NotebookApplicationAddin ()
+		static Gdk.Pixbuf NotebookIcon
 		{
-			notebookIcon = GuiUtils.GetIcon ("tomboy-notebook", 22);
-			newNotebookIcon = GuiUtils.GetIcon ("tomboy-new-notebook", 22);
-			
+			get {
+				if (notebookIcon == null)
+					notebookIcon = GuiUtils.GetIcon ("notebook", 16);
+				return notebookIcon;
+			}
+		}
+
+		static Gdk.Pixbuf NewNotebookIcon
+		{
+			get {
+				if (newNotebookIcon == null)
+					newNotebookIcon = GuiUtils.GetIcon ("notebook-new", 16);
+				return newNotebookIcon;
+			}
 		}
 
 		bool initialized;
@@ -96,7 +107,7 @@ namespace Tomboy.Notebooks
 			if (item != null) {
 				if (item is Gtk.ImageMenuItem) {
 					Gtk.ImageMenuItem imageItem = item as Gtk.ImageMenuItem;
-					(imageItem.Image as Gtk.Image).Pixbuf = notebookIcon;
+					(imageItem.Image as Gtk.Image).Pixbuf = NotebookIcon;
 				}
 				trayNotebookMenu = new Gtk.Menu ();
 				item.Submenu = trayNotebookMenu;
@@ -110,11 +121,22 @@ namespace Tomboy.Notebooks
 			if (imageitem != null) {
 				if (imageitem is Gtk.ImageMenuItem) {
 					Gtk.ImageMenuItem imageItem = imageitem as Gtk.ImageMenuItem;
-					(imageItem.Image as Gtk.Image).Pixbuf = notebookIcon;
+					(imageItem.Image as Gtk.Image).Pixbuf = NotebookIcon;
 				}
 				mainWindowNotebookMenu = new Gtk.Menu ();
 				imageitem.Submenu = mainWindowNotebookMenu;
 				
+#if MAC
+				// Make the menu once, since Shown events aren't working in Mac Menu
+				AddMenuItems (mainWindowNotebookMenu);
+				// Listen to these events to maintain list of Notebooks; watching the TreeStore
+				// takes more work because it's a pre-notify.
+				NotebookManager.NoteAddedToNotebook += delegate (Note o, Notebook args) {
+					AddMenuItems (mainWindowNotebookMenu); };
+				NotebookManager.NoteRemovedFromNotebook += delegate (Note o, Notebook args) {
+					AddMenuItems (mainWindowNotebookMenu); };
+#endif
+
 				mainWindowNotebookMenu.Shown += OnNewNotebookMenuShown;
 				mainWindowNotebookMenu.Hidden += OnNewNotebookMenuHidden;
 			}
@@ -126,6 +148,15 @@ namespace Tomboy.Notebooks
 					(imageItem.Image as Gtk.Image).Pixbuf = ActionManager.newNote;
 				}
 			}
+					
+			foreach (Note note in Tomboy.DefaultNoteManager.Notes) {
+				note.TagAdded += OnTagAdded;
+				note.TagRemoved += OnTagRemoved;
+			}
+					
+			Tomboy.DefaultNoteManager.NoteAdded += OnNoteAdded;
+			Tomboy.DefaultNoteManager.NoteDeleted += OnNoteDeleted;
+				
 			initialized = true;
 		}
 
@@ -190,7 +221,7 @@ namespace Tomboy.Notebooks
 			// Add in the "New Notebook..." menu item
 			Gtk.ImageMenuItem newNotebookMenuItem =
 				new Gtk.ImageMenuItem (Catalog.GetString ("New Note_book..."));
-			newNotebookMenuItem.Image = new Gtk.Image (newNotebookIcon);
+			newNotebookMenuItem.Image = new Gtk.Image (NewNotebookIcon);
 			newNotebookMenuItem.Activated += OnNewNotebookMenuItem;
 			newNotebookMenuItem.ShowAll ();
 			menu.Append (newNotebookMenuItem);
@@ -209,10 +240,16 @@ namespace Tomboy.Notebooks
 					} while (model.IterNext (ref iter) == true);
 				}
 			}
+#if MAC
+			menu.ShowAll ();
+#endif
 		}
 		
 		private void RemoveMenuItems (Gtk.Menu menu)
 		{
+#if MAC
+			menu.HideAll ();
+#endif
 			foreach (Gtk.MenuItem child in menu.Children) {
 				menu.Remove (child);
 			}
@@ -222,5 +259,60 @@ namespace Tomboy.Notebooks
 		{
 			NotebookManager.PromptCreateNewNotebook (null);
 		}
+		
+		/// <summary>
+		/// Handle the addition of note tags through programmatic means,
+		/// such as note sync or the dbus remote control.
+		/// </summary>
+		private void OnTagAdded (Note note, Tag tag)
+		{
+			if (NotebookManager.AddingNotebook)
+				return;
+			
+			string megaPrefix =
+				Tag.SYSTEM_TAG_PREFIX + Notebook.NotebookTagPrefix;
+			if (tag.IsSystem == false
+			    || tag.Name.StartsWith (megaPrefix) == false) {
+				return;
+			}
+			
+			string notebookName =
+				tag.Name.Substring (megaPrefix.Length);
+			
+			Notebook notebook =
+				NotebookManager.GetOrCreateNotebook (notebookName);
+				
+			NotebookManager.FireNoteAddedToNoteBook (note, notebook);
+		}
+			
+		private void OnTagRemoved (Note note, string normalizedTagName)
+		{
+			string megaPrefix =
+				Tag.SYSTEM_TAG_PREFIX + Notebook.NotebookTagPrefix;
+			if (normalizedTagName.StartsWith (megaPrefix) == false)
+				return;
+			
+			string normalizedNotebookName =
+				normalizedTagName.Substring (megaPrefix.Length);
+			
+			Notebook notebook =
+				NotebookManager.GetNotebook (normalizedNotebookName);
+			if (notebook == null)
+				return;
+			
+			NotebookManager.FireNoteRemovedFromNoteBook (note, notebook);
+		}
+		
+		private void OnNoteAdded (object sender, Note note)
+		{
+			note.TagAdded += OnTagAdded;
+			note.TagRemoved += OnTagRemoved;
+		}
+		
+		private void OnNoteDeleted (object sender, Note note)
+		{
+			note.TagAdded -= OnTagAdded;
+			note.TagRemoved -= OnTagRemoved;
+		}
 	}
 }
